﻿BuffInformant = { ['Version'] = tonumber(("$Revision: 9 $"):match("(%d+)")) }

----
----  #####################
----  'External' functions!
----  #####################
----

--[[
	
	Two 'DirectConfigMode's:
	false = change the config directly
	true = change a copy, and use ConfigCancel() or ConficOkay() at the end
	
]]--

function BuffInformant:SetDirectConfigMode(flag)
	if flag == self.DirectConfigMode then
		-- noop
	elseif flag then
		-- if false -> true
		self.Config = nil
	else
		-- if true -> false
		self.Config = {}
		for k,v in pairs(BuffInformant_Config) do
			self.Config[k] = v
		end
	end
	self.DirectConfigMode = flag
	self:Update('list')
end

function BuffInformant:GetPref(key)
	local value
	if self.DirectConfigMode then
		value = BuffInformant_Config[key]
	else
		value = self.Config[key]
	end -- if
	if value == nil then
		value = self.defaults[key]
	end -- if
	return value
end

function BuffInformant:SetPref(key, value)
	if value == self.defaults[key] then
		value = nil
	end
	if self.DirectConfigMode then
		BuffInformant_Config[key] = value
	else
		self.Config[key] = value
	end -- if
end

function BuffInformant:GetColorPref(key)
	local c1, c2, c3 = strsplit(';', self:GetPref(key))
	return tonumber(c1), tonumber(c2), tonumber(c3), 1.0
end

function BuffInformant:SetColorPref(key, r, g, b, a)
	self:SetPref(key, string.format('%1.3f;%1.3f;%1.3f', r, g, b))
end

function BuffInformant:ConfigOkay()
	for k,v in pairs(BuffInformant_Config) do
		BuffInformant_Config[k] = nil
	end
	for k,v in pairs(self.Config) do
		BuffInformant_Config[k] = v
	end
end

function BuffInformant:ConfigCancel()
	for k,v in pairs(self.Config) do
		self.Config[k] = nil
	end
	for k,v in pairs(BuffInformant_Config) do
		self.Config[k] = v
	end
end

function BuffInformant:ConfigDefaults()
	-- only useable when not DirectConfigMode
	if not DirectConfigMode then
		for k,v in pairs(self.Config) do
			if k ~= 'list' then
				self.Config[k] = nil
			end -- if
		end -- for
	end -- if
end

function BuffInformant:GetLocked()
	return self.locked
end

function BuffInformant:SetLocked(value)
	self.locked = value
	if self.locked then
		BuffInformantSplashFrame:SetBackdrop(nil)
		BuffInformantSplashFrame:EnableMouse(false)
	else
		BuffInformantSplashFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background"})
		BuffInformantSplashFrame:EnableMouse(true)
	end -- if
	self:Update('holdtime')
end

function BuffInformant:Update(type)
	if type == 'list' then
		self:ParseBuffList()
	end
	if type == 'list' or type == 'font' or type == 'insertmode' or type == 'holdtime' then
		self:ChangeFont()
	end
	self:UpdateDisplay()
end

----
----  #####################
----    'Internal' stuff!
----  #####################
----

BuffInformant.DirectConfigMode = true
BuffInformant.locked = true
BuffInformant.list = {}

BuffInformant.defaults = {
	['list'] = '',
	['font'] = 'NumberFontNormalHuge',
	['buffstartcolor'] = '0;1;0',
	['buffendcolor'] = '1;1;0',
	['debuffstartcolor'] = '1;0;0',
	['debuffendcolor'] = '1;0;1',
	['holdtime'] = 3,
	['insertmode'] = 'TOP',
	['showicon'] = 'Y',
}

local Stype = {['B'] = 'buff', ['D'] = 'debuff'}
local Sdisplay = {['S'] = 'start', ['E'] = 'end', ['B'] = true, ['T'] = true}

function BuffInformant:OnLoad()
	-- init config
	if type(BuffInformant_Config) ~= 'table' then
		BuffInformant_Config = {}
	end

	-- init some stuff
	self.myname = string.lower(UnitName('player'))
	_, self.myclass = UnitClass('player')
	self.myclass = string.lower(self.myclass)
	self.frame_timeout = 0.2
	self.frame_timeouttimer = self.frame_timeout
	self.frame = CreateFrame('Frame')
	self.frame:SetScript('OnUpdate', function(frame, elapsed)
		self.frame_timeouttimer = self.frame_timeouttimer - elapsed
		if self.frame_timeouttimer <= 0 then
			self.frame_timeouttimer = self.frame_timeout
			self:OnUpdate()
		end -- if
	end)
	self:Update('list')
end

function BuffInformant:OnUpdate()
	local now, update = GetTime(), false

	for _, buff in pairs(self.list) do
		local btype, _, _, icon, _, _, duration, expirationTime, _, _
		if UnitExists(buff.unit) then
				btype, _, _, icon, _, _, duration, expirationTime, _, _ = 'B', UnitBuff(buff.unit, buff.name)
			if not icon then
				btype, _, _, icon, _, _, duration, expirationTime, _, _ = 'D', UnitDebuff(buff.unit, buff.name)
			end -- if
		end -- if

		if icon then
			expirationTime = math.floor(0.5 + expirationTime - now)
			if not buff.active and (buff.display == 'S' or buff.display == 'B') then
				buff.message = 'S'
				buff.expire = now + self:GetPref('holdtime')
				update = true
			elseif buff.display == 'T' and (not buff.active or buff.exptime ~= expirationTime) then
				buff.message = 'T'
				buff.exptime = expirationTime
				buff.duration = duration
				update = true
			end -- if
			buff.active = true
			buff.icon = icon
			buff.type = btype
			buff.enemy = UnitIsEnemy('player', buff.unit)
		else
			if buff.active and (buff.display == 'E' or buff.display == 'B') then
				buff.message = 'E'
				buff.expire = now + self:GetPref('holdtime')
				update = true
			elseif buff.active and buff.message == 'T' then
				buff.message = nil
				update = true
			end -- if
			buff.active = false
		end -- if
		if buff.expire and buff.expire < now then
			buff.expire = nil
			buff.message = nil
		end -- if
	end -- for
	if update then
		self:UpdateDisplay()
	end
end

function BuffInformant:ParseBuffList()
	-- empty the list
	for k, _ in pairs(self.list) do
		self.list[k] = nil
	end -- for

	local list = {strsplit("\n", self:GetPref('list'))}
	local fors, fore, step = 1, #list, 1
	if self:GetPref('insertmode') == 'TOP' then
		fors, fore, step = fore, fors, -step
	end
	for linenr = fors, fore, step do
		local line = list[linenr]
		if strtrim(line) ~= '' and line[1] ~= '#' then
			local ln = {strsplit('||', line)}
			local params = {name = strtrim(ln[1])}
			for i = 2, #ln do
				local key, value = strsplit('=', strtrim(ln[i]))
				if strtrim(ln[i]) ~= '' and value ~= nil and strtrim(value) ~= '' then
					params[strtrim(key)] = strtrim(value)
				end -- if
			end -- for
			
			-- check config
			if
				params.name == '' or
				params.D == nil or not Sdisplay[params.D]
			then
				DEFAULT_CHAT_FRAME:AddMessage('BuffInformant: invalid config in line ' .. linenr, 1, 0, 0)
			elseif not params.C or (string.match(','..string.lower(params.C)..',', ','..self.myname..',') or string.match(','..string.lower(params.C)..',', ','..self.myclass..',')) then
				table.insert(self.list, {
					['name'] = params.name,
					['display'] = params.D,
					['unit'] = params.U or 'player',
					['text'] = params.T or params.name,
					['showicon'] = params.I,
				})
			end -- if
		end -- if
	end -- for
end

function BuffInformant:ChangeFont()
	BuffInformantSplashFrame:SetFontObject(self:GetPref('font'))
	BuffInformantSplashFrame:SetInsertMode(self:GetPref('insertmode'))
	BuffInformantSplashFrame:SetTimeVisible(not self.locked and math.huge or self:GetPref('holdtime'))
end

function BuffInformant:UpdateDisplay()
	if self.locked then
		local now = GetTime()
		BuffInformantSplashFrame:Clear()
		for name, buff in pairs(self.list) do
			local line, showicon = buff.text, buff.showicon or self:GetPref('showicon')
			if buff.message == 'E' then
				line = string.format(string.gsub(AURA_END, '<%%s>', '%%s'), line)
			end -- if
			if showicon == 'Y' and buff.icon then
				line = '|T'..buff.icon..':0|t ' .. line
			end --if
			local btype = buff.type
			if buff.enemy then
				local rev = {['B'] = 'D', ['D'] = 'B'}
				btype = rev[btype]
			end --if
			if buff.message == 'S' or buff.message == 'E' then
				local r, g, b = self:GetColorPref(Stype[btype]..Sdisplay[buff.message]..'color')
				BuffInformantSplashFrame:AddMessage(line, r, g, b, nil, buff.expire - now)
				--self:dump('AddMessage', {name .. ':' .. buff.message, r, g, b, nil, buff.expire - now})
			elseif buff.message == 'T' then
				local m1, t = 0.5, ''
				if type(buff.duration) == 'number' and buff.duration > 0 and type(buff.exptime) == 'number' and buff.exptime >= 0 then
					m1 = buff.exptime / buff.duration
					t = ' - '..buff.exptime..'s'
				end -- if
				local m2 = 1 - m1
				local r1, g1, b1 = self:GetColorPref(Stype[btype]..'startcolor')
				local r2, g2, b2 = self:GetColorPref(Stype[btype]..'endcolor')
				local r, g, b = r1*m1+r2*m2, g1*m1+g2*m2, b1*m1+b2*m2
				BuffInformantSplashFrame:AddMessage(line .. t, r, g, b, nil, math.huge)
				--self:dump('AddMessage', {name .. ':' .. buff.message, r, g, b, nil, 10000000})
			end -- if
		end -- for
	else
		-- demo-mode
		BuffInformantSplashFrame:Clear()
		local r, g, b = self:GetColorPref('debuffendcolor')
		BuffInformantSplashFrame:AddMessage(string.format(string.gsub(AURA_END, '<%%s>', '%%s'), 'Long-Test-Debuff'), r, g, b)
		local r, g, b = self:GetColorPref('buffstartcolor')
		BuffInformantSplashFrame:AddMessage('Extra-Long-Test-Buff', r, g, b)
--		local c1 = self:GetPref('buffstartcolor')
--		local c2 = self:GetPref('buffendcolor')
--		BuffInformantSplashFrame:AddMessage(string.format('|c%02x%02x%02x%02x%s', 128, (c1.r+c2.r)*127.5, (c1.g+c2.g)*127.2, (c1.b+c2.g)*127.5, 'Verstärkung ends in 12s'))
	end -- if
end


--- DEBUGGING...

function BuffInformant:dump(na, va, hide)
        -- for debugging without rock, see near line 50

        -- debug only when needed

        -- output       
        local t
        
        -- name
        if(na ~= nil)then
                t = self:dump(nil, na, true, force)..': '
        else
                t = ''
        end

        -- data
        local ty = type(va)
        if(ty == 'table')then
                t = t .. '{ '
                for k,v in pairs(va) do
                        t = t .. self:dump(k, v, true, force) .. ', '
                end
                t = t .. '}'
        elseif(ty == 'string')then
                t = t .. '\'' .. va .. '\''
        elseif(ty == 'number')then
                t = t .. va
        elseif(ty == 'boolean')then
                t = t .. ((va and 'true') or 'false')
        else
                t = t .. '(' .. ty .. ')'
        end
        
        if(hide)then
                return t
        else
                DEFAULT_CHAT_FRAME:AddMessage('dump: '..t)
        end
end
